@@ -1,13 +1,19 @@
-"""Helpers producing simple HTML tags
+"""Helpers that produce simple HTML tags.
 
-Most helpers have an ``\*\*attrs`` argument to specify additional HTML
+Most helpers have an ``**attrs`` argument to specify additional HTML
 attributes.  A trailing underscore in the name will be deleted; this is 
-especially important for attributes that match Python keywords; e.g.,
-``class_``.  Some helpers handle certain keywords specially; these are noted in
-the helpers' docstrings.
+especially important for attributes that are identical to Python keywords;
+e.g., ``class_``.  Some helpers handle certain keywords specially; these are
+noted in the helpers' docstrings.
+
+To create your own custom tags, see ``webhelpers.html.builder``.
+
+A set of CSS styles complementing these helpers is in
+``webhelpers/public/stylesheets/webhelpers.css``.
 """
 
 import datetime
+import logging
 import os
 import re
 import urllib
@@ -15,13 +21,15 @@ import urlparse
 
 from webhelpers import containers
 from webhelpers.html import escape, HTML, literal, url_escape
+import webhelpers.media as media
+from webhelpers.misc import NotGiven
 
 __all__ = [
            # Form tags
            "form", "end_form", 
            "text", "textarea", "hidden", "file", "password", 
            "checkbox", "radio", "submit",
-           "select", "Options", "Option",
+           "select", "Options", "Option", "OptGroup",
            "ModelTags", "title", "required_legend",
            # hyperlinks
            "link_to", "link_to_if", "link_to_unless",
@@ -29,16 +37,19 @@ __all__ = [
            "th_sortable",
            # Other non-form tags
            "ol", "ul", "image", "BR",
-           # Head tags
+           # Head tags and document type
            "stylesheet_link", "javascript_link", "auto_discovery_link",
+           "Doctype", "xml_declaration",
            # Utility functions
-           "convert_boolean_attrs",
+           "css_classes", "convert_boolean_attrs",
            ]
 
+log = logging.getLogger(__name__)
+
 NL = literal("\n")
 BR = literal("<br />\n")
 
-def form(url, method="post", multipart=False, **attrs):
+def form(url, method="post", multipart=False, hidden_fields=None, **attrs):
     """An open tag for a form that will submit to ``url``.
 
     You must close the form yourself by calling ``end_form()`` or outputting
@@ -46,17 +57,29 @@ def form(url, method="post", multipart=False, **attrs):
     
     Options:
 
-    ``multipart``
-        If set to True, the enctype is set to "multipart/form-data".
-        You must set it to true when uploading files, or the browser will
-        submit the filename rather than the file.
-
     ``method``
         The method to use when submitting the form, usually either 
         "GET" or "POST". If "PUT", "DELETE", or another verb is used, a
         hidden input with name _method is added to simulate the verb
         over POST.
     
+    ``multipart``
+        If set to True, the enctype is set to "multipart/form-data".
+        You must set it to true when uploading files, or the browser will
+        submit the filename rather than the file.
+
+    ``hidden_fields``
+        Additional hidden fields to add to the beginning of the form.  It may
+        be a dict or an iterable of key-value tuples. This is implemented by
+        calling the object's ``.items()`` method if it has one, or just
+        iterating the object.  (This will successfuly get multiple values for
+        the same key in WebOb MultiDict objects.)
+
+    Because input tags must be placed in a block tag rather than directly
+    inside the form, all hidden fields will be put in a 
+    '<div style="display:none">'.  The style prevents the <div> from being
+    displayed or affecting the layout.
+
     Examples:
 
     >>> form("/submit")
@@ -64,20 +87,39 @@ def form(url, method="post", multipart=False, **attrs):
     >>> form("/submit", method="get")
     literal(u'<form action="/submit" method="get">')
     >>> form("/submit", method="put")
-    literal(u'<form action="/submit" method="post"><input name="_method" type="hidden" value="put" />')
+    literal(u'<form action="/submit" method="post"><div style="display:none">\\n<input name="_method" type="hidden" value="put" />\\n</div>\\n')
     >>> form("/submit", "post", multipart=True) 
     literal(u'<form action="/submit" enctype="multipart/form-data" method="post">')
+
+    Changed in WebHelpers 1.0b2: add <div> and ``hidden_fields`` arg.
+
+    Changed in WebHelpers 1.2: don't add an "id" attribute to hidden tags
+    generated by this helper; they clash if there are multiple forms on the
+    page.
     """
+    fields = []
+    attrs["action"] = url
     if multipart:
         attrs["enctype"] = "multipart/form-data"
-    method_tag = literal("")
     if method.lower() in ['post', 'get']:
         attrs['method'] = method
     else:
         attrs['method'] = "post"
-        method_tag = HTML.input(type="hidden", name="_method", value=method)
-    attrs["action"] = url
-    return HTML.form(method_tag, _closed=False, **attrs)
+        field = hidden("_method", method, id=None)
+        fields.append(field)
+    if hidden_fields is not None:
+        try:
+            it = hidden_fields.items()
+        except AttributeError:
+            it = hidden_fields
+        for name, value in it:
+            field = hidden(name, value, id=None)
+            fields.append(field)
+    if fields:
+        div = HTML.div(style="display:none", _nl=True, *fields)
+    else:
+        div = None
+    return HTML.form(div, _closed=False, **attrs)
 
 
 def end_form():
@@ -91,16 +133,20 @@ def end_form():
     return literal("</form>")
 
 
-def text(name, value=None, id=None, **attrs):
+def text(name, value=None, id=NotGiven, type="text", **attrs):
     """Create a standard text field.
     
     ``value`` is a string, the content of the text field.
 
     ``id`` is the HTML ID attribute, and should be passed as a keyword
     argument.  By default the ID is the same as the name filtered through
-    ``_make_safe_id_component()``.  Pass the empty string ("") to suppress the
+    ``_make_safe_id_component()``.  Pass None to suppress the
     ID attribute entirely.
 
+    ``type`` is the input field type, normally "text". You can override it
+    for HTML 5 input fields that don't have their own helper; e.g.,
+    "search", "email", "date".
+
     
     Options:
     
@@ -112,22 +158,33 @@ def text(name, value=None, id=None, **attrs):
         will allow the user to enter.
     
     The remaining keyword args will be standard HTML attributes for the tag.
+
+    Example, a text input field::
+
+        >>> text("address")
+        literal(u'<input id="address" name="address" type="text" />')
+
+    HTML 5 example, a color picker:
+
+        >>> text("color", type="color")
+        literal(u'<input id="color" name="color" type="color" />')
     
     """
-    _set_input_attrs(attrs, "text", name, value)
+    _set_input_attrs(attrs, type, name, value)
     _set_id_attr(attrs, id, name)
     convert_boolean_attrs(attrs, ["disabled"])
     return HTML.input(**attrs)
 
 
-def hidden(name, value=None, **attrs):
+def hidden(name, value=None, id=NotGiven, **attrs):
     """Create a hidden field.
     """
     _set_input_attrs(attrs, "hidden", name, value)
+    _set_id_attr(attrs, id, name)
     return HTML.input(**attrs)
 
 
-def file(name, value=None, id=None, **attrs):
+def file(name, value=None, id=NotGiven, **attrs):
     """Create a file upload field.
     
     If you are using file uploads then you will also need to set the 
@@ -144,7 +201,7 @@ def file(name, value=None, id=None, **attrs):
     return HTML.input(**attrs)
 
 
-def password(name, value=None, id=None, **attrs):
+def password(name, value=None, id=NotGiven, **attrs):
     """Create a password field.
     
     Takes the same options as ``text()``.
@@ -155,7 +212,7 @@ def password(name, value=None, id=None, **attrs):
     return HTML.input(**attrs)
 
 
-def textarea(name, content="", id=None, **attrs):
+def textarea(name, content="", id=NotGiven, **attrs):
     """Create a text input area.
     
     Example::
@@ -169,7 +226,7 @@ def textarea(name, content="", id=None, **attrs):
     return HTML.textarea(content, **attrs)
 
 
-def checkbox(name, value="1", checked=False, label=None, id=None, **attrs):
+def checkbox(name, value="1", checked=False, label=None, id=NotGiven, **attrs):
     """Create a check box.
 
     Arguments:
@@ -183,10 +240,9 @@ def checkbox(name, value="1", checked=False, label=None, id=None, **attrs):
 
     ``id`` is the HTML ID attribute, and should be passed as a keyword
     argument.  By default the ID is the same as the name filtered through
-    ``_make_safe_id_component()``.  Pass the empty string ("") to suppress the
+    ``_make_safe_id_component()``.  Pass None to suppress the
     ID attribute entirely.
 
-d
     The following HTML attributes may be set by keyword argument:
 
     * ``disabled`` - If true, checkbox will be grayed out.
@@ -242,14 +298,14 @@ def radio(name, value, checked=False, label=None, **attrs):
     return widget
 
 
-def submit(name, value, id=None, **attrs):
+def submit(name, value, id=NotGiven, **attrs):
     """Create a submit button with the text ``value`` as the caption."""
     _set_input_attrs(attrs, "submit", name, value)
     _set_id_attr(attrs, id, name)
     return HTML.input(**attrs)
 
 
-def select(name, selected_values, options, id=None, **attrs):
+def select(name, selected_values, options, id=NotGiven, **attrs):
     """Create a dropdown selection box.
 
     * ``name`` -- the name of this control.
@@ -261,10 +317,12 @@ def select(name, selected_values, options, id=None, **attrs):
       pairs.  The label will be shown on the form; the option will be returned
       to the application if that option is chosen.  If you pass a string or int
       instead of a 2-tuple, it will be used for both the value and the label.
+      If the `value` is a tuple or a list, it will be added as an optgroup,
+      with `label` as label.
 
     ``id`` is the HTML ID attribute, and should be passed as a keyword
     argument.  By default the ID is the same as the name.  filtered through
-    ``_make_safe_id_component()``.  Pass the empty string ("") to suppress the
+    ``_make_safe_id_component()``.  Pass None to suppress the
     ID attribute entirely.
 
 
@@ -299,6 +357,9 @@ def select(name, selected_values, options, id=None, **attrs):
         literal(u'<select id="currency" name="currency">\\n<option selected="selected" value="">Please choose ...</option>\\n<option value="$">Dollar</option>\\n<option value="DKK">Kroner</option>\\n</select>')
         >>> select("privacy", 3L, [(1, "Private"), (2, "Semi-public"), (3, "Public")])
         literal(u'<select id="privacy" name="privacy">\\n<option value="1">Private</option>\\n<option value="2">Semi-public</option>\\n<option selected="selected" value="3">Public</option>\\n</select>')
+        >>> select("recipients", None, [([("u1", "User1"), ("u2", "User2")], "Users"), ([("g1", "Group1"), ("g2", "Group2")], "Groups")])
+        literal(u'<select id="recipients" name="recipients">\\n<optgroup label="Users">\\n<option value="u1">User1</option>\\n<option value="u2">User2</option>\\n</optgroup>\\n<optgroup label="Groups">\\n<option value="g1">Group1</option>\\n<option value="g2">Group2</option>\\n</optgroup>\\n</select>')
+        
     """
     _set_id_attr(attrs, id, name)
     attrs["name"] = name
@@ -319,12 +380,22 @@ def select(name, selected_values, options, id=None, **attrs):
     if not isinstance(options, Options):
         options = Options(options)
     html_options = []
+    # Create the options structure
+    def gen_opt(val, label):
+        if val in selected_values:
+            return HTML.option(label, value=val, selected="selected")
+        else:
+            return HTML.option(label, value=val)
+    # Loop options and create tree (if optgroups presents)
     for opt in options:
-       if opt.value in selected_values:
-           opt = HTML.option(opt.label, value=opt.value, selected="selected")
-       else:
-           opt = HTML.option(opt.label, value=opt.value)
-       html_options.append(opt)
+        if isinstance(opt, OptGroup):
+            optgroup_options = []
+            for subopt in opt.options:
+                optgroup_options.append(gen_opt(subopt.value, subopt.label))
+            optgroup = HTML.optgroup(NL, NL.join(optgroup_options), NL, label=opt.label)
+            html_options.append(optgroup)
+        else:
+            html_options.append(gen_opt(opt.value, opt.label))
     return HTML.select(NL, NL.join(html_options), NL, **attrs)
 
 
@@ -455,6 +526,11 @@ class ModelTags(object):
         equals ``checked_value``.  On form submission the value will be 
         ``checked_value`` if the button was selected, or ``""`` otherwise.
 
+        In case of a ModelTags object that is created from scratch
+        (e.g. ``new_employee=ModelTags(None)``) the option that should
+        be checked can be set by the 'default' parameter. As in:
+        ``new_employee.radio('status', checked_value=7, default=7)``
+
         The control's 'id' attribute will be modified as follows:
 
         1. If not specified but an 'id_format' was given to the constructor,
@@ -553,10 +629,23 @@ class Option(object):
     """
     __slots__ = ("value", "label")
 
+
+    def __repr__(self):
+        return str((self.value, self.label))
     def __init__(self, value, label):
         self.value = value
         self.label = label
 
+class OptGroup(object):
+    """A container for Options"""
+    __slots__ = ('options', 'label')
+    def __init__(self, label, options):
+        self.options = Options(options)
+        self.label = label
+    def __repr__(self):
+        classname = self.__class__.__name__
+        data = [x for x in self.options]
+        return "%s(%s, %s)" % (classname, data, repr(self.label))
 
 class Options(tuple):
     """A tuple of ``Option`` objects for the ``select()`` helper.
@@ -566,6 +655,7 @@ class Options(tuple):
     convenient.
 
     This class has multiple jobs:
+
     - Canonicalize the options given to ``select()`` into a consistent format.
     - Avoid canonicalizing the same data multiple times.  It subclasses tuple
       rather than a list to guarantee that nonconformant elements won't be 
@@ -584,26 +674,30 @@ class Options(tuple):
     >>> opts[2].label
     u'B'
     """
-
     def __new__(class_, options):
         opts = []
         for opt in options:
-            if not isinstance(opt, Option):
-                if isinstance(opt, (list, tuple)):
-                    value, label = opt[:2]
-                else:
-                    value = label = opt
-                if not isinstance(value, unicode):
-                    value = unicode(value)
-                if not isinstance(label, unicode):  # Preserves literal.
-                    label = unicode(label)
-                opt = Option(value, label)
+            if isinstance(opt, (Option, OptGroup)):
+                opts.append(opt)
+                continue
+            if isinstance(opt, (list, tuple)):
+                value, label = opt[:2]
+                if isinstance(value, (list, tuple)):  # It's an optgroup
+                    opts.append(OptGroup(label, value))
+                    continue
+            else:
+                value = label = opt
+            if not isinstance(value, unicode):
+                value = unicode(value)
+            if not isinstance(label, unicode):  # Preserves literal.
+                label = unicode(label)
+            opt = Option(value, label)
             opts.append(opt)
         return super(Options, class_).__new__(class_, opts)
 
     def __repr__(self):
         classname = self.__class__.__name__
-        data = [(x.value, x.label) for x in self]
+        data = [x for x in self]
         return "%s(%s)" % (classname, data)
         
     def values(self):
@@ -622,7 +716,7 @@ def title(title, required=False, label_for=None):
 
     ``title`` -- the name of the field; e.g., "First Name".
 
-    ``required `` -- if true, append a "*" to the title and use the
+    ``required`` -- if true, append a \*" to the title and use the
     'required' HTML format (see example); otherwise use the 'not
     required' format.
 
@@ -708,7 +802,8 @@ def link_to_if(condition, label, url='', **attrs):
         return label
 
 def link_to_unless(condition, label, url='', **attrs):
-    """Same as ``link_to`` but return just the label if the condition is true.
+    """The opposite of ``link_to``. Return just the label if the condition is 
+    true.
     """
     if not condition:
         return link_to(label, url, **attrs)
@@ -836,7 +931,8 @@ def _list(tag, items, default, attrs, li_attrs):
     return getattr(HTML, tag)(content, **attrs)
     
 
-def image(url, alt, width=None, height=None, **attrs):
+def image(url, alt, width=None, height=None, path=None, use_pil=False, 
+    **attrs):
     """Return an image tag for the specified ``source``.
 
     ``url``
@@ -851,10 +947,25 @@ def image(url, alt, width=None, height=None, **attrs):
         alt tag completely, pass None.
 
     ``width``
-        The width of the image, default is not included
+        The width of the image, default is not included.
 
     ``height``
-        The height of the image, default is not included
+        The height of the image, default is not included.
+
+    ``path``
+        Calculate the width and height based on the image file at ``path`` if
+        possible. May not be specified if ``width`` or ``height`` is 
+        specified. The results are also written to the debug log for
+        troubleshooting.
+
+    ``use_pil``
+        If true, calcuate the image dimensions using the Python Imaging 
+        Library, which must be installed. Otherwise use a pure Python
+        algorithm which understands fewer image formats and may be less
+        accurate. This flag controls whether
+        ``webhelpers.media.get_dimensions_pil`` or
+        ``webhelpers.media.get_dimensions`` is called. It has no effect if
+        ``path`` is not specified.
         
     Examples::
 
@@ -872,18 +983,71 @@ def image(url, alt, width=None, height=None, **attrs):
 
         >>> image("/icons/icon.gif", None, width=16)
         literal(u'<img alt="" src="/icons/icon.gif" width="16" />')
-        
     """
     if not alt:
         alt = ""
-    if width is not None:
+    if width is not None or height is not None:
         attrs['width'] = width
-    if height is not None:
         attrs['height'] = height
+        if path:
+            raise TypeError(
+                "can't specify path if width and height are specified")
+    elif path:
+        if use_pil:
+            result = media.get_dimensions_pil(path)
+            msg = "using PIL"
+        else:
+            result = media.get_dimensions(path)
+            msg = "not using PIL"
+        abspath = os.path.abspath(path)
+        log.debug("image size is %s for '%s' (%s)", result, abspath, msg)
+        attrs['width'] = result[0]
+        attrs['height'] = result[1]
     return HTML.img(src=url, alt=alt, **attrs)
 
+#### Non-tag utilities
+
+def css_classes(value_condition_pairs):
+    """Add CSS classes to a tag programmatically.
+
+    This helper is meant to be used as the ``class_`` argument to a tag helper.
+
+    The argument is an iterable of ``(class, condition)`` pairs, where each
+    ``class`` is a string and ``condition`` is a boolean.  The function 
+    returns a space-separated list of classes whose conditions were true.
+
+    If all conditions are false, return ``None``.  This tells the caller to
+    suppress the "class" attribute entirely.
+
+    Examples::
+
+        >>> arg = [("first", False), ("even", True)]
+        >>> HTML.td("My content.", class_=css_classes(arg))
+        literal(u'<td class="even">My content.</td>')
+        >>> arg = [("first", True), ("even", True)]
+        >>> HTML.td("My content.", class_=css_classes(arg))
+        literal(u'<td class="first even">My content.</td>')
+        >>> arg = [("first", False), ("even", False)]
+        >>> HTML.td("My content.", class_=css_classes(arg))
+        literal(u'<td>My content.</td>')
+
+ 
+    """
+    classes = []
+    for value, condition in value_condition_pairs:
+        if condition:    # Or ``if condition and value``
+            classes.append(value)
+    if not classes:
+        return None
+    return " ".join(classes)
+
+
 #### Tags for the HTML head
 
+    #### Private methods
+    def _make_doctype(self, type, uri, dtd):
+        return literal('<!DOCTYPE %s PUBLIC "%s" "%s">') % (type, uri, dtd)
+
 def javascript_link(*urls, **attrs):
     """Return script include tags for the specified javascript URLs.
     
@@ -982,6 +1146,80 @@ def auto_discovery_link(url, feed_type="rss", **attrs):
     attrs.setdefault("title", title)
     return HTML.link(rel="alternate", type=feed_type, href=url, **attrs)
 
+#### Document type and XML declaration
+
+class Doctype(object):
+    """Document type declarations for HTML and XHTML."""
+
+    def html5(self):
+        """Create a <!DOCTYPE> for HTML 5.
+        
+        Usage::
+
+            >>> Doctype().html5()
+            literal(u'<!doctype html>')
+        """
+        return literal("<!doctype html>")
+
+    def xhtml1(self, subtype="transitional", version="1.0"):
+        """Create a <!DOCTYPE> for XHTML 1.
+
+           Usage::
+
+               >>> Doctype().xhtml1()
+               literal(u'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')
+               >>> Doctype().xhtml1("strict")
+               literal(u'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">')
+               >>> Doctype().xhtml1("frameset")
+               literal(u'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">')
+        """
+        if subtype in ["transitional", "loose"]:
+            name = "Transitional"
+            dtd = "transitional"
+        else:
+            name = subtype.capitalize()
+            dtd = subtype.lower()
+        uri = "-//W3C//DTD XHTML %s %s//EN" % (version, name)
+        dtd = "http://www.w3.org/TR/xhtml1/DTD/xhtml1-%s.dtd" % dtd
+        return self._make_doctype("html", uri, dtd)
+
+    def html4(self, subtype="transitional", version="4.01"):
+        """Create a <!DOCTYPE> for HTML 4.
+
+           Usage:
+
+               >>> Doctype().html4()
+               literal(u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">')
+               >>> Doctype().html4("strict")
+               literal(u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN" "http://www.w3.org/TR/html4/strict.dtd">')
+               >>> Doctype().html4("frameset")
+               literal(u'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" "http://www.w3.org/TR/html4/frameset.dtd">')
+        """
+        if subtype in ["transitional", "loose"]:
+            name = "Transitional"
+            dtd = "loose"
+        else:
+            name = subtype.capitalize()
+            dtd = subtype.lower()
+        uri = "-//W3C//DTD HTML %s %s//EN" % (version, name)
+        dtd = "http://www.w3.org/TR/html4/%s.dtd" % dtd
+        return self._make_doctype("HTML", uri, dtd)
+
+    #### Private methods
+    def _make_doctype(self, type, uri, dtd):
+        return literal('<!DOCTYPE %s PUBLIC "%s" "%s">') % (type, uri, dtd)
+
+
+def xml_declaration(version="1.0", encoding="utf-8"):
+    """Create an XML declaration.
+
+       Usage::
+
+           >>> xml_declaration()
+           literal(u'<?xml version="1.0" encoding="utf-8" ?>')
+    """
+    return literal('<?xml version="%s" encoding="%s" ?>') % (version, encoding)
+
 
 
 ########## INTERNAL FUNCTIONS ##########
@@ -1011,11 +1249,16 @@ def _set_input_attrs(attrs, type, name, value):
     attrs["name"] = name
     attrs["value"] = value
 
-def _set_id_attr(attrs, id, name):
-    if id is None:
+def _set_id_attr(attrs, id_arg, name):
+    if "id_" in attrs:
+        if id_arg is not NotGiven:
+            raise TypeError("can't pass both 'id' and 'id_' args to helper")
+        attrs["id"] = attrs.pop("id_")
+    elif id_arg is NotGiven:
         attrs["id"] = _make_safe_id_component(name)
-    elif id != "":
-        attrs["id"] = id
+    elif id_arg is not None and id_arg != "":
+        attrs["id"] = id_arg
+    # Else id_arg is None or "", so do nothing.
 
 def _make_safe_id_component(idstring):
     """Make a string safe for including in an id attribute.
